@@ -1,6 +1,5 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999-2004, 2007-2012, 2014-2017 Free Software
-   Foundation, Inc.
+   Copyright (C) 1999-2021 Free Software Foundation, Inc.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -36,6 +35,7 @@
 #include <mailutils/header.h>
 #include <mailutils/iterator.h>
 #include <mailutils/kwd.h>
+#include <mailutils/locus.h>
 #include <mailutils/sieve.h>
 #include <mailutils/list.h>
 #include <mailutils/locker.h>
