@@ -1,5 +1,5 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 2002-2019 Free Software Foundation, Inc.
+   Copyright (C) 2002-2021 Free Software Foundation, Inc.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -150,6 +150,11 @@ _base64_decoder (void *xd MU_ARG_UNUSED,
 	    data[i++] = tmp;
 	  else if (*(iptr-1) == '=')
 	    {
+	      if (pad == 3)
+		{
+		  iobuf->errcode = MU_ERR_BASE64;
+		  return mu_filter_failure;
+		}
 	      data[i++] = 0;
 	      pad++;
 	    }
