@@ -1,5 +1,5 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 2010-2020 Free Software Foundation, Inc.
+   Copyright (C) 2010-2021 Free Software Foundation, Inc.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -30,7 +30,7 @@ mu_property_invalidate (mu_property_t prop)
   
   if (!prop)
     return EINVAL;
-  rc = _mu_property_init (prop);
+  rc = mu_property_init (prop);
   if (rc)
     return rc;
   if (prop->_prop_flags & MU_PROP_FILL)
