@@ -1,6 +1,5 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999-2001, 2005-2007, 2009-2012, 2014-2017 Free
-   Software Foundation, Inc.
+   Copyright (C) 1999-2021 Free Software Foundation, Inc.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -50,6 +49,7 @@ static struct format_record format_table[] = {
   { "mbox", &mu_mbox_record },
   { "mh",   &mu_mh_record },
   { "maildir", &mu_maildir_record },
+  { "dotmail", &mu_dotmail_record },
   { "pop",  &mu_pop_record },
   { "pops",  &mu_pops_record },
   { "imap", &mu_imap_record },
@@ -214,6 +214,7 @@ mu_scm_init ()
   mu_scm_mime_init ();
   mu_scm_debug_port_init ();
   mu_scm_debug_init ();
+  mu_scm_filter_init ();
   
 #include "mu_scm.x"
   
