@@ -1,6 +1,5 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999-2002, 2005-2012, 2014-2016 Free Software
-   Foundation, Inc.
+   Copyright (C) 1999-2021 Free Software Foundation, Inc.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -18,6 +17,8 @@
 
 #include <mailutils/sieve.h>
 #include <mailutils/assoc.h>
+#include <mailutils/locus.h>
+#include <mailutils/yyloc.h>
 #include <setjmp.h>
 #include <string.h>
 #include <regex.h>
@@ -38,13 +39,6 @@ typedef union
   unsigned unum;
 } sieve_op_t;
 
-struct mu_locus_range
-{
-  struct mu_locus beg;
-  struct mu_locus end;
-};
-  
-#define YYLTYPE struct mu_locus_range
 
 #define MU_SV_SAVED_ERR_STATE 0x01
 #define MU_SV_SAVED_DBG_STATE 0x02
@@ -62,8 +56,11 @@ enum mu_sieve_state
 struct mu_sieve_machine
 {
   /* Static data */
-  struct mu_locus locus;     /* Approximate location in the code */
-
+  struct mu_locus_range locus; /* Approximate location in the code */
+  int changeloc:1;             /* If set during code generation phase, the
+				  _mu_i_sv_instr_locus will be emitted before
+				  next instruction */
+  
   mu_list_t memory_pool;     /* Pool of allocated memory objects */
   mu_list_t destr_list;      /* List of destructor functions */
 
@@ -97,7 +94,8 @@ struct mu_sieve_machine
   regmatch_t *match_buf;     /* Offsets of parenthesized groups */
   size_t match_count;        /* Actual number of elements used in match_buf */
   size_t match_max;          /* Total number of elements available in match_buf */
-
+  mu_list_t init_var;        /* List of variable initializers */
+  
   /* Call environment */
   const char *identifier;    /* Name of action or test being executed */
   size_t argstart;           /* Index of the first argument in valspace */
@@ -118,9 +116,9 @@ struct mu_sieve_machine
   /* Stream state info */
   int state_flags;
   int err_mode;
-  struct mu_locus err_locus;
+  struct mu_locus_range err_locus;
   int dbg_mode;
-  struct mu_locus dbg_locus;
+  struct mu_locus_range dbg_locus;
   
   /* User supplied data */
   mu_stream_t errstream;
@@ -144,6 +142,7 @@ enum mu_sieve_node_type
     mu_sieve_node_anyof,
     mu_sieve_node_allof,
     mu_sieve_node_not,
+    mu_sieve_node_end,
   };
 
 struct mu_sieve_node
@@ -170,13 +169,24 @@ struct mu_sieve_node
     } command;
   } v;
 };
+
+struct mu_sieve_node_list
+{
+  struct mu_sieve_node *head, *tail;
+};
+
+struct mu_sieve_variable_initializer
+{
+  char *name;
+  char *value;
+};
 
 int mu_sieve_yyerror (const char *s);
 int mu_sieve_yylex (void);
 
 int mu_i_sv_lex_begin (const char *name);
 int mu_i_sv_lex_begin_string (const char *buf, int bufsize,
-			      const char *fname, int line);
+			      struct mu_locus_point const *pt);
 void mu_i_sv_lex_finish (void);
 
 extern mu_sieve_machine_t mu_sieve_machine;
@@ -196,8 +206,7 @@ void _mu_i_sv_instr_not (mu_sieve_machine_t mach);
 void _mu_i_sv_instr_branch (mu_sieve_machine_t mach);
 void _mu_i_sv_instr_brz (mu_sieve_machine_t mach);
 void _mu_i_sv_instr_brnz (mu_sieve_machine_t mach);
-void _mu_i_sv_instr_source (mu_sieve_machine_t mach);
-void _mu_i_sv_instr_line (mu_sieve_machine_t mach);
+void _mu_i_sv_instr_locus (mu_sieve_machine_t mach);
 
 int mu_i_sv_load_add_dir (mu_sieve_machine_t mach, const char *name);
 
@@ -238,7 +247,6 @@ void mu_i_sv_lint_command (struct mu_sieve_machine *mach,
 
 size_t  mu_i_sv_string_create (mu_sieve_machine_t mach, char *str);
 
-char *mu_i_sv_id_canon (mu_sieve_machine_t mach, char const *name);
 size_t mu_i_sv_id_num (mu_sieve_machine_t mach, char const *name);
 char *mu_i_sv_id_str (mu_sieve_machine_t mach, size_t n);
 void mu_i_sv_free_idspace (mu_sieve_machine_t mach);
@@ -247,4 +255,7 @@ void mu_i_sv_copy_variables (mu_sieve_machine_t child,
 			     mu_sieve_machine_t parent);
 int mu_i_sv_expand_variables (char const *input, size_t len,
 			      char **exp, void *data);
+void mu_i_sv_init_variables (mu_sieve_machine_t mach);
+
+
 
