@@ -1,6 +1,5 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999-2003, 2005, 2007, 2009-2012, 2014-2016 Free
-   Software Foundation, Inc.
+   Copyright (C) 1999-2021 Free Software Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -33,7 +32,7 @@ static int print_stream (mu_stream_t, mu_stream_t);
 static int display_message (mu_message_t, msgset_t *msgset, void *closure);
 static int display_submessage (struct mime_descend_closure *closure,
 			       void *data);
-static int get_content_encoding (mu_header_t hdr, char **value);
+static void get_content_encoding (mu_header_t hdr, char **value);
 static void run_metamail (const char *mailcap, mu_message_t mesg);
 
 int
@@ -74,7 +73,7 @@ display_message (mu_message_t mesg, msgset_t *msgset, void *arg)
   mime_descend (&mclos, display_submessage, NULL);
 
   /* Mark enclosing message as read */
-  if (mu_mailbox_get_message (mbox, msgset->msg_part[0], &mesg) == 0)
+  if (mu_mailbox_get_message (mbox, msgset_msgno (msgset), &mesg) == 0)
     util_mark_read (mesg);
 
   return 0;
@@ -122,41 +121,22 @@ display_headers (mu_stream_t out, mu_message_t mesg,
     }
 }
 
-void
-format_msgset (mu_stream_t str, const msgset_t *msgset, size_t *count)
-{
-  int i;
-  mu_stream_stat_buffer stat;
-
-  if (count)
-    mu_stream_set_stat (str, MU_STREAM_STAT_MASK (MU_STREAM_STAT_OUT),
-			stat);
-  mu_stream_printf (str, "%lu", (unsigned long) msgset->msg_part[0]);
-  for (i = 1; i < msgset->npart; i++)
-    mu_stream_printf (str, "[%lu", (unsigned long) msgset->msg_part[i]);
-  for (i = 1; i < msgset->npart; i++)
-    mu_stream_printf (str, "]");
-  if (count)
-    {
-      *count = stat[MU_STREAM_STAT_OUT];
-      mu_stream_set_stat (str, 0, NULL);
-    }
-}
-
 static void
 display_part_header (mu_stream_t str, const msgset_t *msgset,
 		     const char *type, const char *encoding)
 {
   int size = util_screen_columns () - 3;
   unsigned int i;
-
+  char *msp;
+  
   mu_stream_printf (str, "+");
   for (i = 0; (int)i <= size; i++)
     mu_stream_printf (str, "-");
   mu_stream_printf (str, "+");
   mu_stream_printf (str, "\n");
-  mu_stream_printf (str, "%s", _("| Message="));
-  format_msgset (str, msgset, NULL);
+  msp = msgset_str (msgset);
+  mu_stream_printf (str, _("| Message=%s"), msp);
+  free (msp);
   mu_stream_printf (str, "\n");
 
   mu_stream_printf (str, _("| Type=%s\n"), type);
@@ -173,7 +153,6 @@ mime_descend (struct mime_descend_closure *closure,
 	      mime_descend_fn fun, void *data)
 {
   int status = 0;
-  size_t nparts = 0;
   mu_header_t hdr = NULL;
   char *type;
   char *encoding;
@@ -182,7 +161,7 @@ mime_descend (struct mime_descend_closure *closure,
 
   mu_message_get_header (closure->message, &hdr);
   util_get_hdr_value (hdr, MU_HEADER_CONTENT_TYPE, &type);
-  if (type == NULL)
+  if (!type)
     type = mu_strdup ("text/plain");
   get_content_encoding (hdr, &encoding);
 
@@ -196,36 +175,101 @@ mime_descend (struct mime_descend_closure *closure,
   if (ismime)
     {
       unsigned int j;
-
-      mu_message_get_num_parts (closure->message, &nparts);
-
-      for (j = 1; j <= nparts; j++)
+      size_t nparts;
+      
+      status = mu_message_get_num_parts (closure->message, &nparts);
+      if (status)
 	{
-	  mu_message_t message = NULL;
-
-	  if (mu_message_get_part (closure->message, j, &message) == 0)
+	  mu_diag_funcall (MU_DIAG_ERR, "mu_message_get_num_parts", NULL,
+			   status);
+	}
+      else
+	{
+	  for (j = 1; j <= nparts; j++)
 	    {
-	      msgset_t *set = msgset_expand (msgset_dup (closure->msgset),
-					     msgset_make_1 (j));
-	      subclosure.msgset = set;
-	      subclosure.message = message;
-	      status = mime_descend (&subclosure, fun, data);
-	      msgset_free (set);
-	      if (status)
-		break;
+	      mu_message_t message = NULL;
+
+	      if (mu_message_get_part (closure->message, j, &message) == 0)
+		{
+		  msgset_t *set = msgset_expand (msgset_dup (closure->msgset),
+						 msgset_make_1 (j));
+		  subclosure.msgset = set;
+		  subclosure.message = message;
+		  status = mime_descend (&subclosure, fun, data);
+		  msgset_free (set);
+		  if (status)
+		    break;
+		}
 	    }
 	}
     }
-  else if (mu_c_strncasecmp (type, "message/rfc822", strlen (type)) == 0)
+  else if (mu_c_strncasecmp (type, "message/rfc822", 14) == 0)
     {
       mu_message_t submsg = NULL;
 
-      if (mu_message_unencapsulate (closure->message, &submsg, NULL) == 0)
+      switch (mu_message_unencapsulate (closure->message, &submsg, NULL))
 	{
+	case 0:
 	  subclosure.hints = MDHINT_SELECTED_HEADERS;
 	  subclosure.msgset = closure->msgset;
 	  subclosure.message = submsg;
 	  status = mime_descend (&subclosure, fun, data);
+	  break;
+
+	case MU_ERR_INVALID_EMAIL:
+	  /*
+	   * The mu_message_unencapsulate function returns this code
+	   * if it was unable to parse the message body as a RFC822
+	   * message (this code is propagated from mu_stream_to_message,
+	   * which does the actual work).
+	   *
+	   * If the enclosing messgae(part) is of message/digest type, it
+	   * is possible that messages are packed into it without MIME
+	   * headers.  That violates RFC 1341, but such digests are
+	   * reported to exist (re. email conversation with Karl on
+	   * 2020-08-07, <202008070138.0771cfHn003390@freefriends.org>).
+	   *
+	   * Try to see whether the part has one of the normal RFC822 headers
+	   * and if so treat it as the message.  Otherwise, treat it as
+	   * text/plain.
+	   * 
+	   * FIXME: Do all this only if the upper-level message is of
+	   * message/digest type.
+	   */
+	  {
+	    char *names[] = { "From", "To", "Subject" };
+	    mu_header_t hdr;
+
+	    if (mu_message_get_header (closure->message, &hdr) == 0 &&
+		mu_header_sget_firstof (hdr, names, NULL, NULL) == 0)
+	      {
+		mu_stream_t str;
+		if (mu_message_get_streamref (closure->message, &str) == 0)
+		  {
+		    status = mu_stream_to_message (str, &submsg);
+		    mu_stream_unref (str);
+		    if (status == 0)
+		      {
+			mu_header_t subhdr;
+			if (mu_message_get_header (submsg, &subhdr) == 0)
+			  {
+			    mu_header_remove (subhdr,
+					      MU_HEADER_CONTENT_TYPE, 1);
+			    subclosure.hints = MDHINT_SELECTED_HEADERS;
+			    subclosure.msgset = closure->msgset;
+			    subclosure.message = submsg;
+			    status = mime_descend (&subclosure, fun, data);
+			    break;
+			  }
+		      }
+		  }
+	      }
+	  }
+	  /* FALLTHROUGH */
+	  
+	default:
+	  /* Treat as text/plain */
+	  status = fun (closure, data);
 	}
     }
   else
@@ -245,7 +289,7 @@ display_submessage (struct mime_descend_closure *closure, void *data)
 {
   char *tmp;
   
-  if (mailvar_get (&tmp, "metamail", mailvar_type_string, 0) == 0)
+  if (mailvar_get (&tmp, mailvar_name_metamail, mailvar_type_string, 0) == 0)
     {
       /* If `metamail' is set to a string, treat it as command line
 	 of external metamail program. */
@@ -281,18 +325,14 @@ display_submessage (struct mime_descend_closure *closure, void *data)
       
       /* If `metamail' is set to true, enable internal mailcap
 	 support */
-      if (mailvar_get (NULL, "metamail", mailvar_type_boolean, 0) == 0)
+      if (mailvar_is_true (mailvar_name_metamail))
 	{
 	  char *no_ask = NULL;
-	  int debug = 0;
-	  
-	  mailvar_get (&no_ask, "mimenoask", mailvar_type_string, 0);
-	  if (mailvar_get (&debug, "verbose", mailvar_type_boolean, 0) == 0
-	      && debug)
-	    debug = 9;
 	  
+	  mailvar_get (&no_ask, mailvar_name_mimenoask, mailvar_type_string, 0);
 	  builtin_display = display_stream_mailcap (NULL, stream, hdr, no_ask,
-						    interactive, 0, debug);
+						    interactive, 0,
+						    MU_DEBCAT_APP);
 	}
       
       if (builtin_display)
@@ -337,10 +377,10 @@ print_stream (mu_stream_t stream, mu_stream_t out)
   return 0;
 }
 
-static int
+static void
 get_content_encoding (mu_header_t hdr, char **value)
 {
-  char *encoding = NULL;
+  char *encoding;
   util_get_hdr_value (hdr, MU_HEADER_CONTENT_TRANSFER_ENCODING, &encoding);
   if (encoding == NULL || *encoding == '\0')
     {
@@ -349,7 +389,6 @@ get_content_encoding (mu_header_t hdr, char **value)
       encoding = mu_strdup ("7bit"); /* Default.  */
     }
   *value = encoding;
-  return 0;
 }
 
 /* Run `metamail' program MAILCAP_CMD on the message MESG */
@@ -405,13 +444,14 @@ run_metamail (const char *mailcap_cmd, mu_message_t mesg)
 	  char *no_ask;
 	  
 	  setenv ("METAMAIL_PAGER", getenv ("PAGER"), 0);
-	  if (mailvar_get (&no_ask, "mimenoask", mailvar_type_string, 0))
+	  if (mailvar_get (&no_ask, mailvar_name_mimenoask,
+			   mailvar_type_string, 0))
 	    setenv ("MM_NOASK", no_ask, 1);
 	  
 	  status = mu_message_get_streamref (mesg, &stream);
 	  if (status)
 	    {
-	      mu_error ("mu_message_get_stream: %s", mu_strerror (status));
+	      mu_error ("mu_message_get_streamref: %s", mu_strerror (status));
 	      break;
 	    }
 
@@ -447,3 +487,88 @@ run_metamail (const char *mailcap_cmd, mu_message_t mesg)
   sigaction (SIGQUIT, &savequit, NULL);
   sigprocmask (SIG_SETMASK, &savemask, NULL);
 }
+
+/* print_message_body(msg, out, stat)
+ *
+ * Prints the body of the message MSG to the output stream OUT.  If
+ * the Content-Transfer-Encoding header is set, the body is decoded.
+ * If stat is not NULL, it must point to an array of two size_t.
+ * Upon return stat[0] will contain the number of bytes and stat[1]
+ * the number of newline characters written to the output.
+ */
+int
+print_message_body (mu_message_t msg, mu_stream_t out, size_t *stat)
+{
+  int rc;
+  mu_header_t hdr;
+  mu_body_t body;
+  mu_stream_t d_stream;
+  mu_stream_t stream;
+  char *encoding = NULL;
+  mu_stream_stat_buffer sb;
+  
+  rc = mu_message_get_header (msg, &hdr);
+  if (rc)
+    {
+      mu_diag_funcall (MU_DIAG_ERROR, "mu_message_get_header",
+		       NULL, rc);
+      return rc;
+    }
+      
+  rc = mu_message_get_body (msg, &body);
+  if (rc)
+    {
+      mu_diag_funcall (MU_DIAG_ERROR, "mu_message_get_body",
+		       NULL, rc);
+      return rc;
+    }
+  
+  rc = mu_body_get_streamref (body, &stream);
+  if (rc)
+    {
+      mu_diag_funcall (MU_DIAG_ERROR, "mu_body_get_streamref",
+		       NULL, rc);
+      return rc;
+    }
+
+  util_get_hdr_value (hdr, MU_HEADER_CONTENT_TRANSFER_ENCODING, &encoding);
+  if (encoding == NULL || *encoding == '\0')
+    /* No need to filter */;
+  else if ((rc = mu_filter_create (&d_stream, stream, encoding,
+				   MU_FILTER_DECODE, MU_STREAM_READ)) == 0)
+    {
+      mu_stream_unref (stream);
+      stream = d_stream;
+    }
+  else
+    {
+      mu_diag_funcall (MU_DIAG_ERROR, "mu_message_get_body",
+		       encoding, rc);
+      /* FIXME: continue anyway? */
+    }
+
+  if (stat)
+    {
+      mu_stream_set_stat (stream,
+			  MU_STREAM_STAT_MASK (MU_STREAM_STAT_IN) |
+			  MU_STREAM_STAT_MASK (MU_STREAM_STAT_INLN),
+			  sb);
+    }
+  rc = mu_stream_copy (out, stream, 0, NULL);
+
+  mu_stream_destroy (&stream);
+  free (encoding);
+
+  if (rc)
+    {
+      mu_diag_funcall (MU_DIAG_ERROR, "mu_stream_copy",
+		       encoding, rc);
+    }
+  else if (stat)
+    {
+      stat[0] = sb[MU_STREAM_STAT_IN];
+      stat[1] = sb[MU_STREAM_STAT_INLN];
+    }
+  
+  return rc;
+}
