@@ -1,9 +1,9 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999, 2001, 2002, 2003, 2005 Free Software Foundation, Inc.
+   Copyright (C) 1999-2021 Free Software Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2, or (at your option)
+   the Free Software Foundation; either version 3, or (at your option)
    any later version.
 
    GNU Mailutils is distributed in the hope that it will be useful,
@@ -12,9 +12,7 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with GNU Mailutils; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
-   MA 02110-1301 USA */
+   along with GNU Mailutils.  If not, see <http://www.gnu.org/licenses/>. */
 
 #include "mail.h"
 
@@ -46,7 +44,7 @@ mail_previous (int argc, char **argv)
 
       if (rc)
 	{
-	  util_error (_("No applicable message"));
+	  mu_error (_("No applicable message"));
 	  return 1;
 	}
     }
@@ -56,14 +54,14 @@ mail_previous (int argc, char **argv)
       int rc = msgset_parse (argc, argv, MSG_NODELETED|MSG_SILENT, &list);
       if (!rc)
 	{
-	  n = list->msg_part[0];
+	  n = msgset_msgno (list);
 	  msgset_free (list);
 	  if (util_get_message (mbox, n, &msg))
 	    return 1;
 	}
       else
 	{
-	  util_error (_("No applicable message"));
+	  mu_error (_("No applicable message"));
 	  return 1;
 	}
     }
