@@ -1,5 +1,5 @@
 #  GNU Mailutils -- a suite of utilities for electronic mail
-#  Copyright (C) 2009-2012, 2014-2015 Free Software Foundation, Inc.
+#  Copyright (C) 2009-2021 Free Software Foundation, Inc.
 #
 #  This library is free software; you can redistribute it and/or
 #  modify it under the terms of the GNU Lesser General Public
@@ -15,7 +15,6 @@
 #  Public License along with this library.  If not, see
 #  <http://www.gnu.org/licenses/>.
 
-import types
 from mailutils.c_api import registrar
 
 def register_format (name=None):
@@ -42,8 +41,7 @@ def register_format (name=None):
     """
     if name == None:
         registrar.register_format ()
-    elif isinstance (name, types.TupleType) \
-      or isinstance (name, types.ListType):
+    elif isinstance(name, tuple) or isinstance(name, list):
         for n in name:
             registrar.register_format (n)
     else:
